// Generated by CoffeeScript 2.2.3
(function() {
  //!/usr/bin/env coffee
  var $, Daemon, Fs, cmd, configFile, createBasePath, doInit, echo, exists, exit_soon, expandPath, sendServerCmd, socketFile, verbose, warn;

  ({$, cmd, echo, warn, verbose} = require('../common'));

  Fs = require('fs');

  ({exists, socketFile, configFile, createBasePath, expandPath} = require('../files'));

  Daemon = require('../daemon');

  if (cmd.help || cmd.h) {
    console.log("shepherd <start|stop|restart|status|add|remove|enable|disable>");
    process.exit(0);
  }

  exit_soon = (code = 0, ms = 100) => {
    return setTimeout((() => {
      return process.exit(code);
    }), ms);
  };

  doInit = function(cb) {
    var defaultsFile;
    defaultsFile = process.cwd() + "/.shepherd/defaults";
    if (exists(configFile) && !(cmd.f || cmd.force)) {
      echo(`Configuration already exists (${configFile})`);
      if (typeof cb === "function") {
        cb(null, false);
      }
    } else {
      verbose("Checking for defaults file:", defaultsFile);
      if (exists(defaultsFile)) {
        echo("Applying default config...");
        Fs.copyFile(expandPath(defaultsFile), expandPath(configFile), (err) => {
          return typeof cb === "function" ? cb(null, true) : void 0;
        });
      } else {
        createBasePath(".", cb);
      }
    }
    return null;
  };

  sendServerCmd = (_cmd, cb) => {
    var Actions, Net, Tnet, action, on_error, retryConnect;
    ({Actions} = require('../actions'));
    if (!(action = Actions[_cmd])) {
      return warn("No such action:", _cmd);
    }
    Net = require('net');
    Tnet = require('../util/tnet');
    on_error = function(err) {
      return warn("socket error", $.debugStack(err));
    };
    (retryConnect = function() {
      var connectStart, socket;
      connectStart = Date.now();
      socket = Net.connect({
        path: expandPath(socketFile)
      });
      socket.on('close', () => {
        return typeof cb === "function" ? cb() : void 0;
      });
      socket.on('error', function(err) { // probably daemon is not running, should start it
        if (err.code === 'ENOENT') {
          if (_cmd === 'start') {
            Daemon.doStart(false);
            return setTimeout(retryConnect, 3000);
          } else {
            echo("Status: offline.");
            return exit_soon(1);
          }
        } else if (err.code === 'EADDRNOTAVAIL') {
          echo("Status: offline.");
          return exit_soon(1);
        } else {
          return on_error(err);
        }
      });
      socket.on('connect', function() {
        var bytes, err, msg;
        socket._connectLatency = Date.now() - connectStart;
        try {
          msg = action.toMessage(cmd);
          if (cmd._.length > 1 && !(('group' in cmd) || ('instance' in cmd))) {
            if (/\w/.test(cmd._[1])) {
              msg.auto = cmd._[1];
            }
          }
          bytes = $.TNET.stringify(msg);
        } catch (error) {
          err = error;
          return on_error(err);
        }
        socket.write(bytes, function() {
          var timeout;
          if (typeof action.onConnect === "function") {
            action.onConnect(socket);
          }
          if ('onResponse' in action) {
            timeout = $.delay(1000, function() {
              warn("Timed-out waiting for a response from the daemon.");
              return socket.end();
            });
            Tnet.read_stream(socket, function(item) {
              timeout.cancel();
              return action.onResponse(item, socket);
            });
          } else {
            socket.end();
          }
          return null;
        });
        return null;
      });
      return null;
    })();
    return null;
  };

  switch (cmd._[0]) {
    case 'init':
      return doInit(exit_soon);
    case 'up':
      Daemon.doStart(false);
      $.delay(1000, () => {
        return sendServerCmd('status');
      });
      break;
    case 'down':
      return Daemon.doStop(true);
    default:
      sendServerCmd(cmd._[0], () => {
        var ref;
        if ((ref = cmd._[0]) === 'start' || ref === 'stop' || ref === 'enable' || ref === 'disable' || ref === 'add' || ref === 'remove' || ref === 'scale' || ref === 'replace') {
          return setTimeout((() => {
            return sendServerCmd('status');
          }), 300);
        }
      });
  }

}).call(this);
